var __defProp = Object.defineProperty;
var __defProps = Object.defineProperties;
var __getOwnPropDescs = Object.getOwnPropertyDescriptors;
var __getOwnPropSymbols = Object.getOwnPropertySymbols;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __propIsEnum = Object.prototype.propertyIsEnumerable;
var __defNormalProp = (obj, key, value) => key in obj ? __defProp(obj, key, { enumerable: true, configurable: true, writable: true, value }) : obj[key] = value;
var __spreadValues = (a, b) => {
  for (var prop in b || (b = {}))
    if (__hasOwnProp.call(b, prop))
      __defNormalProp(a, prop, b[prop]);
  if (__getOwnPropSymbols)
    for (var prop of __getOwnPropSymbols(b)) {
      if (__propIsEnum.call(b, prop))
        __defNormalProp(a, prop, b[prop]);
    }
  return a;
};
var __spreadProps = (a, b) => __defProps(a, __getOwnPropDescs(b));
var __objRest = (source, exclude) => {
  var target = {};
  for (var prop in source)
    if (__hasOwnProp.call(source, prop) && exclude.indexOf(prop) < 0)
      target[prop] = source[prop];
  if (source != null && __getOwnPropSymbols)
    for (var prop of __getOwnPropSymbols(source)) {
      if (exclude.indexOf(prop) < 0 && __propIsEnum.call(source, prop))
        target[prop] = source[prop];
    }
  return target;
};
var __async = (__this, __arguments, generator) => {
  return new Promise((resolve, reject) => {
    var fulfilled = (value) => {
      try {
        step(generator.next(value));
      } catch (e) {
        reject(e);
      }
    };
    var rejected = (value) => {
      try {
        step(generator.throw(value));
      } catch (e) {
        reject(e);
      }
    };
    var step = (x) => x.done ? resolve(x.value) : Promise.resolve(x.value).then(fulfilled, rejected);
    step((generator = generator.apply(__this, __arguments)).next());
  });
};

// src/index.ts
export * from "@supabase-cache-helpers/postgrest-shared";
export * from "@supabase-cache-helpers/postgrest-filter";
export * from "@supabase-cache-helpers/postgrest-fetcher";

// src/cache/use-delete-item.ts
import {
  deleteItem
} from "@supabase-cache-helpers/postgrest-mutate";
import { useSWRConfig as useSWRConfig3 } from "swr";

// src/lib/constants.ts
var KEY_PREFIX = "postgrest";
var POSTGREST_FILTER_KEY_PREFIX = "postgrest-filter";
var KEY_SEPARATOR = "$";
var INFINITE_PREFIX = "$inf$";
var INFINITE_KEY_PREFIX = "page";

// src/lib/create-key-getter.ts
import {
  isPostgrestHasMorePaginationResponse,
  isPostgrestPaginationResponse
} from "@supabase-cache-helpers/postgrest-shared";
var createKeyGetter = (query, pageSize) => {
  if (!query)
    return () => null;
  return (pageIndex, previousPageData) => {
    if (previousPageData && (isPostgrestHasMorePaginationResponse(previousPageData) && !previousPageData.data.length || isPostgrestPaginationResponse(previousPageData) && !previousPageData.length))
      return null;
    const cursor = pageIndex * pageSize;
    return query.range(cursor, cursor + pageSize);
  };
};

// src/lib/decode.ts
var decode = (key) => {
  if (typeof key !== "string")
    return null;
  const isInfinite = key.startsWith(INFINITE_PREFIX);
  let parsedKey = key.replace(INFINITE_PREFIX, "");
  const isPostgrestKey = parsedKey.startsWith(`${KEY_PREFIX}${KEY_SEPARATOR}`);
  if (!isPostgrestKey) {
    return null;
  }
  parsedKey = parsedKey.replace(`${KEY_PREFIX}${KEY_SEPARATOR}`, "");
  const [
    pagePrefix,
    schema,
    table,
    queryKey,
    bodyKey,
    count,
    head,
    orderByKey
  ] = parsedKey.split(KEY_SEPARATOR);
  const params = new URLSearchParams(queryKey);
  const limit = params.get("limit");
  const offset = params.get("offset");
  const countValue = count.replace("count=", "");
  return {
    limit: limit ? Number(limit) : void 0,
    offset: offset ? Number(offset) : void 0,
    bodyKey,
    count: countValue === "null" ? null : countValue,
    isHead: head === "head=true",
    isInfinite,
    key,
    isInfiniteKey: pagePrefix === INFINITE_KEY_PREFIX,
    queryKey,
    schema,
    table,
    orderByKey
  };
};

// src/lib/encode.ts
var encode = (parser, isInfinite) => {
  var _a;
  return [
    KEY_PREFIX,
    isInfinite ? INFINITE_KEY_PREFIX : "null",
    parser.schema,
    parser.table,
    parser.queryKey,
    (_a = parser.bodyKey) != null ? _a : "null",
    `count=${parser.count}`,
    `head=${parser.isHead}`,
    parser.orderByKey
  ].join(KEY_SEPARATOR);
};

// src/lib/middleware.ts
import { PostgrestParser } from "@supabase-cache-helpers/postgrest-filter";
import { isPostgrestBuilder } from "@supabase-cache-helpers/postgrest-shared";
var middleware = (useSWRNext) => {
  return (key, fetcher, config) => {
    if (!fetcher)
      throw new Error("No fetcher provided");
    if (key !== null && !isPostgrestBuilder(key)) {
      throw new Error("Key is not a PostgrestBuilder");
    }
    return useSWRNext(
      key ? encode(new PostgrestParser(key), false) : null,
      () => fetcher(key),
      config
    );
  };
};
var infiniteMiddleware = (useSWRInfiniteNext) => {
  return (keyFn, fetcher, config) => {
    return useSWRInfiniteNext(
      (index, previousPageData) => {
        const query = keyFn(index, previousPageData);
        if (!query)
          return null;
        if (!isPostgrestBuilder(query)) {
          throw new Error("Key is not a PostgrestBuilder");
        }
        return encode(new PostgrestParser(query), true);
      },
      typeof fetcher === "function" ? (query) => fetcher(query) : fetcher,
      config
    );
  };
};

// src/lib/use-postgrest-filter-cache.ts
import {
  encodeObject,
  PostgrestFilter
} from "@supabase-cache-helpers/postgrest-filter";
import { useSWRConfig } from "swr";
var usePostgrestFilterCache = () => {
  const { cache } = useSWRConfig();
  return (query, opts) => {
    const key = [
      POSTGREST_FILTER_KEY_PREFIX,
      query,
      opts ? encodeObject(opts) : null
    ].filter(Boolean).join(KEY_SEPARATOR);
    const cacheData = cache.get(key);
    if (cacheData && cacheData.data instanceof PostgrestFilter) {
      return cacheData.data;
    }
    const filter = PostgrestFilter.fromQuery(query, opts);
    cache.set(key, { data: filter });
    return filter;
  };
};

// src/lib/use-queries-for-table-loader.ts
import { useSWRConfig as useSWRConfig2 } from "swr";
var useQueriesForTableLoader = (table) => {
  const { cache } = useSWRConfig2();
  const getPostgrestFilter = usePostgrestFilterCache();
  return () => Array.from(cache.keys()).reduce((prev, curr) => {
    const decodedKey = decode(curr);
    if ((decodedKey == null ? void 0 : decodedKey.table) === table) {
      prev.push(getPostgrestFilter(decodedKey.queryKey).params);
    }
    return prev;
  }, []);
};

// src/lib/mutable-keys.ts
var getMutableKeys = (keys) => {
  return keys.filter((k) => {
    const decoded = decode(k);
    return decoded && (decoded.isInfinite || !decoded.isInfiniteKey);
  });
};

// src/cache/use-delete-item.ts
function useDeleteItem(opts) {
  const { mutate, cache } = useSWRConfig3();
  const getPostgrestFilter = usePostgrestFilterCache();
  return (input) => __async(this, null, function* () {
    return yield deleteItem(
      __spreadValues({
        input
      }, opts),
      {
        cacheKeys: getMutableKeys(Array.from(cache.keys())),
        getPostgrestFilter,
        mutate,
        decode
      }
    );
  });
}

// src/cache/use-upsert-item.ts
import {
  upsertItem
} from "@supabase-cache-helpers/postgrest-mutate";
import { useSWRConfig as useSWRConfig4 } from "swr";
function useUpsertItem(opts) {
  const { mutate, cache } = useSWRConfig4();
  const getPostgrestFilter = usePostgrestFilterCache();
  return (input) => __async(this, null, function* () {
    return yield upsertItem(
      __spreadValues({
        input
      }, opts),
      {
        cacheKeys: getMutableKeys(Array.from(cache.keys())),
        getPostgrestFilter,
        mutate,
        decode
      }
    );
  });
}

// src/mutate/use-delete-mutation.ts
import { buildDeleteFetcher } from "@supabase-cache-helpers/postgrest-fetcher";
import {
  getTable
} from "@supabase-cache-helpers/postgrest-shared";
import useMutation from "swr/mutation";

// src/mutate/use-random-key.ts
import { useId } from "react";
var PREFIX = "random-mutation-key";
var useRandomKey = () => {
  const key = useId();
  return [PREFIX, key].join(KEY_SEPARATOR);
};

// src/mutate/use-delete-mutation.ts
function useDeleteMutation(qb, primaryKeys, query, opts) {
  const key = useRandomKey();
  const queriesForTable = useQueriesForTableLoader(getTable(qb));
  const deleteItem2 = useDeleteItem({
    primaryKeys,
    table: getTable(qb),
    schema: qb.schema,
    opts
  });
  return useMutation(
    key,
    (_0, _1) => __async(this, [_0, _1], function* (_, { arg }) {
      var _a;
      const result = yield buildDeleteFetcher(qb, primaryKeys, {
        query: query != null ? query : void 0,
        queriesForTable,
        disabled: opts == null ? void 0 : opts.disableAutoQuery
      })(arg);
      if (result) {
        deleteItem2(result == null ? void 0 : result.normalizedData);
      }
      return (_a = result == null ? void 0 : result.userQueryData) != null ? _a : null;
    }),
    opts
  );
}

// src/mutate/use-insert-mutation.ts
import { buildInsertFetcher } from "@supabase-cache-helpers/postgrest-fetcher";
import {
  getTable as getTable2
} from "@supabase-cache-helpers/postgrest-shared";
import useMutation2 from "swr/mutation";

// src/mutate/get-user-response.ts
function truthy(value) {
  return !!value;
}
var getUserResponse = (d) => {
  if (!d)
    return d;
  return d.map((r) => r.userQueryData).filter(truthy);
};

// src/mutate/use-insert-mutation.ts
function useInsertMutation(qb, primaryKeys, query, opts) {
  const key = useRandomKey();
  const queriesForTable = useQueriesForTableLoader(getTable2(qb));
  const upsertItem2 = useUpsertItem({
    primaryKeys,
    table: getTable2(qb),
    schema: qb.schema,
    opts
  });
  return useMutation2(
    key,
    (_0, _1) => __async(this, [_0, _1], function* (key2, { arg }) {
      const result = yield buildInsertFetcher(qb, {
        query: query != null ? query : void 0,
        queriesForTable,
        disabled: opts == null ? void 0 : opts.disableAutoQuery
      })(arg);
      if (result) {
        Promise.all(
          (result != null ? result : []).map(
            (d) => __async(this, null, function* () {
              return yield upsertItem2(d.normalizedData);
            })
          )
        );
      }
      return getUserResponse(result);
    }),
    opts
  );
}

// src/mutate/use-update-mutation.ts
import { buildUpdateFetcher } from "@supabase-cache-helpers/postgrest-fetcher";
import {
  getTable as getTable3
} from "@supabase-cache-helpers/postgrest-shared";
import useSWRMutation from "swr/mutation";
function useUpdateMutation(qb, primaryKeys, query, opts) {
  const key = useRandomKey();
  const queriesForTable = useQueriesForTableLoader(getTable3(qb));
  const upsertItem2 = useUpsertItem({
    primaryKeys,
    table: getTable3(qb),
    schema: qb.schema,
    opts
  });
  return useSWRMutation(
    key,
    (_0, _1) => __async(this, [_0, _1], function* (_, { arg }) {
      var _a;
      const result = yield buildUpdateFetcher(qb, primaryKeys, {
        query: query != null ? query : void 0,
        queriesForTable,
        disabled: opts == null ? void 0 : opts.disableAutoQuery
      })(arg);
      if (result == null ? void 0 : result.normalizedData) {
        upsertItem2(result == null ? void 0 : result.normalizedData);
      }
      return (_a = result == null ? void 0 : result.userQueryData) != null ? _a : null;
    }),
    opts
  );
}

// src/mutate/use-upsert-mutation.ts
import { buildUpsertFetcher } from "@supabase-cache-helpers/postgrest-fetcher";
import {
  getTable as getTable4
} from "@supabase-cache-helpers/postgrest-shared";
import useMutation3 from "swr/mutation";
function useUpsertMutation(qb, primaryKeys, query, opts) {
  const key = useRandomKey();
  const queriesForTable = useQueriesForTableLoader(getTable4(qb));
  const upsertItem2 = useUpsertItem({
    primaryKeys,
    table: getTable4(qb),
    schema: qb.schema,
    opts
  });
  return useMutation3(
    key,
    (_0, _1) => __async(this, [_0, _1], function* (_, { arg }) {
      const result = yield buildUpsertFetcher(qb, {
        query: query != null ? query : void 0,
        queriesForTable,
        disabled: opts == null ? void 0 : opts.disableAutoQuery
      })(arg);
      if (result) {
        Promise.all(
          result.map(
            (d) => __async(this, null, function* () {
              return yield upsertItem2(d.normalizedData);
            })
          )
        );
      }
      return getUserResponse(result);
    }),
    opts
  );
}

// src/query/use-infinite-query.ts
import { createPaginationFetcher } from "@supabase-cache-helpers/postgrest-fetcher";
import useSWRInfinite from "swr/infinite";
function useInfiniteQuery(query, config) {
  var _a, _b, _c;
  return useSWRInfinite(
    createKeyGetter(query, (_a = config == null ? void 0 : config.pageSize) != null ? _a : 20),
    createPaginationFetcher(
      query,
      (key) => {
        const decodedKey = decode(key);
        if (!decodedKey) {
          throw new Error("Not a SWRPostgrest key");
        }
        return {
          limit: decodedKey.limit,
          offset: decodedKey.offset
        };
      },
      (_b = config == null ? void 0 : config.pageSize) != null ? _b : 20
    ),
    __spreadProps(__spreadValues({}, config), {
      use: [
        ...(_c = config == null ? void 0 : config.use) != null ? _c : [],
        infiniteMiddleware
      ]
    })
  );
}

// src/query/use-infinite-scroll-query.ts
import { createPaginationHasMoreFetcher } from "@supabase-cache-helpers/postgrest-fetcher";
import { useMemo } from "react";
import useSWRInfinite2 from "swr/infinite";
function useInfiniteScrollQuery(query, config) {
  var _a, _b, _c;
  const _d = useSWRInfinite2(
    createKeyGetter(query, (_a = config == null ? void 0 : config.pageSize) != null ? _a : 20),
    createPaginationHasMoreFetcher(
      query,
      (key) => {
        const decodedKey = decode(key);
        if (!decodedKey) {
          throw new Error("Not a SWRPostgrest key");
        }
        return {
          limit: decodedKey.limit,
          offset: decodedKey.offset
        };
      },
      (_b = config == null ? void 0 : config.pageSize) != null ? _b : 20
    ),
    __spreadProps(__spreadValues({}, config), {
      use: [
        ...(_c = config == null ? void 0 : config.use) != null ? _c : [],
        infiniteMiddleware
      ]
    })
  ), { data, setSize, size } = _d, rest = __objRest(_d, ["data", "setSize", "size"]);
  const { data: flatData, hasMore } = useMemo(() => {
    return {
      data: (data != null ? data : []).flatMap((p) => p.data),
      hasMore: Array.isArray(data) && data.length > 0 && data[data.length - 1].hasMore
    };
  }, [data]);
  return __spreadValues({
    data: flatData,
    size,
    setSize,
    loadMore: hasMore ? () => setSize(size + 1) : null
  }, rest);
}

// src/query/use-pagination-query.ts
import { createPaginationHasMoreFetcher as createPaginationHasMoreFetcher2 } from "@supabase-cache-helpers/postgrest-fetcher";
import { useMemo as useMemo2, useState } from "react";
import useSWRInfinite3 from "swr/infinite";
function usePaginationQuery(query, config) {
  var _a, _b, _c, _e;
  const _d = useSWRInfinite3(
    createKeyGetter(query, (_a = config == null ? void 0 : config.pageSize) != null ? _a : 20),
    createPaginationHasMoreFetcher2(
      query,
      (key) => {
        const decodedKey = decode(key);
        if (!decodedKey) {
          throw new Error("Not a SWRPostgrest key");
        }
        return {
          limit: decodedKey.limit,
          offset: decodedKey.offset
        };
      },
      (_b = config == null ? void 0 : config.pageSize) != null ? _b : 20
    ),
    __spreadProps(__spreadValues({}, config), {
      use: [
        ...(_c = config == null ? void 0 : config.use) != null ? _c : [],
        infiniteMiddleware
      ]
    })
  ), { data, setSize, size } = _d, rest = __objRest(_d, ["data", "setSize", "size"]);
  const [currentPageIndex, setCurrentPageIndex] = useState(0);
  const { data: parsedData, hasMore } = useMemo2(() => {
    return {
      data: (data != null ? data : []).map((p) => p.data),
      hasMore: Array.isArray(data) && data.length > 0 && data[data.length - 1].hasMore
    };
  }, [data]);
  return __spreadValues({
    pages: parsedData,
    currentPage: parsedData ? (_e = parsedData[currentPageIndex]) != null ? _e : [] : [],
    pageIndex: currentPageIndex,
    setPage: (idx) => {
      if (idx > size - 1) {
        setSize(idx + 1);
      }
      setCurrentPageIndex(idx);
    },
    nextPage: hasMore || currentPageIndex < size - 1 ? () => {
      if (currentPageIndex === size - 1) {
        setSize((size2) => size2 + 1);
      }
      setCurrentPageIndex((page) => page + 1);
    } : null,
    previousPage: currentPageIndex > 0 ? () => setCurrentPageIndex((current) => current - 1) : null
  }, rest);
}

// src/query/use-query.ts
import {
  isPostgrestBuilder as isPostgrestBuilder2
} from "@supabase-cache-helpers/postgrest-shared";
import useSWR from "swr";
function useQuery(query, config) {
  var _a, _c;
  const _b = useSWR(
    query,
    (q) => __async(this, null, function* () {
      if (isPostgrestBuilder2(q)) {
        q = q.throwOnError();
      }
      return yield q;
    }),
    __spreadProps(__spreadValues({}, config), {
      use: [...(_a = config == null ? void 0 : config.use) != null ? _a : [], middleware]
    })
  ), { data } = _b, rest = __objRest(_b, ["data"]);
  return __spreadValues({ data: data == null ? void 0 : data.data, count: (_c = data == null ? void 0 : data.count) != null ? _c : null }, rest);
}

// src/subscribe/use-subscription-query.ts
import { loadQuery } from "@supabase-cache-helpers/postgrest-fetcher";
import {
  REALTIME_LISTEN_TYPES,
  REALTIME_POSTGRES_CHANGES_LISTEN_EVENT
} from "@supabase/supabase-js";
import { useEffect, useState as useState2 } from "react";
function useSubscriptionQuery(client, channelName, filter, primaryKeys, query, opts) {
  const [status, setStatus] = useState2();
  const [channel, setChannel] = useState2();
  const queriesForTable = useQueriesForTableLoader(filter.table);
  const deleteItem2 = useDeleteItem({
    primaryKeys,
    table: filter.table,
    schema: filter.schema,
    opts
  });
  const upsertItem2 = useUpsertItem({
    primaryKeys,
    table: filter.table,
    schema: filter.schema,
    opts
  });
  useEffect(() => {
    if (!client)
      return;
    const c = client.channel(channelName).on(
      REALTIME_LISTEN_TYPES.POSTGRES_CHANGES,
      filter,
      (payload) => __async(this, null, function* () {
        var _a;
        let data = (_a = payload.new) != null ? _a : payload.old;
        const selectQuery = loadQuery({ queriesForTable, query });
        if (payload.eventType !== REALTIME_POSTGRES_CHANGES_LISTEN_EVENT.DELETE && selectQuery) {
          const qb = client.from(payload.table).select(selectQuery.selectQuery);
          for (const pk of primaryKeys) {
            qb.eq(pk.toString(), data[pk]);
          }
          const res = yield qb.single();
          if (res.data)
            data = res.data;
        }
        if (payload.eventType === REALTIME_POSTGRES_CHANGES_LISTEN_EVENT.INSERT || payload.eventType === REALTIME_POSTGRES_CHANGES_LISTEN_EVENT.UPDATE) {
          yield upsertItem2(data);
        } else if (payload.eventType === REALTIME_POSTGRES_CHANGES_LISTEN_EVENT.DELETE) {
          yield deleteItem2(data);
        }
        if (opts == null ? void 0 : opts.callback) {
          opts.callback(__spreadProps(__spreadValues({}, payload), {
            data
          }));
        }
      })
    ).subscribe((status2) => setStatus(status2));
    setChannel(c);
    return () => {
      if (c)
        c.unsubscribe();
    };
  }, []);
  return { channel, status };
}

// src/subscribe/use-subscription.ts
import {
  REALTIME_LISTEN_TYPES as REALTIME_LISTEN_TYPES2,
  REALTIME_POSTGRES_CHANGES_LISTEN_EVENT as REALTIME_POSTGRES_CHANGES_LISTEN_EVENT2
} from "@supabase/supabase-js";
import { useEffect as useEffect2, useState as useState3 } from "react";
function useSubscription(channel, filter, primaryKeys, opts) {
  const [status, setStatus] = useState3();
  const deleteItem2 = useDeleteItem({
    primaryKeys,
    table: filter.table,
    schema: filter.schema,
    opts
  });
  const upsertItem2 = useUpsertItem({
    primaryKeys,
    table: filter.table,
    schema: filter.schema,
    opts
  });
  useEffect2(() => {
    if (!channel)
      return;
    const c = channel.on(
      REALTIME_LISTEN_TYPES2.POSTGRES_CHANGES,
      filter,
      (payload) => __async(this, null, function* () {
        if (payload.eventType === REALTIME_POSTGRES_CHANGES_LISTEN_EVENT2.INSERT || payload.eventType === REALTIME_POSTGRES_CHANGES_LISTEN_EVENT2.UPDATE) {
          yield upsertItem2(payload.new);
        } else if (payload.eventType === REALTIME_POSTGRES_CHANGES_LISTEN_EVENT2.DELETE) {
          yield deleteItem2(payload.old);
        }
        if (opts == null ? void 0 : opts.callback) {
          opts.callback(__spreadValues({}, payload));
        }
      })
    ).subscribe((status2) => setStatus(status2));
    return () => {
      if (c)
        c.unsubscribe();
    };
  }, []);
  return { status };
}
export {
  INFINITE_KEY_PREFIX,
  INFINITE_PREFIX,
  KEY_PREFIX,
  KEY_SEPARATOR,
  POSTGREST_FILTER_KEY_PREFIX,
  createKeyGetter,
  decode,
  encode,
  infiniteMiddleware,
  middleware,
  useDeleteItem,
  useDeleteMutation,
  useInfiniteQuery,
  useInfiniteScrollQuery,
  useInsertMutation,
  usePaginationQuery,
  usePostgrestFilterCache,
  useQueriesForTableLoader,
  useQuery,
  useSubscription,
  useSubscriptionQuery,
  useUpdateMutation,
  useUpsertItem,
  useUpsertMutation
};
//# sourceMappingURL=index.mjs.map